local lib = LibStub("LibAuraInfo-1.0");
if(not lib) then error("API.lua for LibAuraInfo needs library loaded."); return; end

local debugPrint = lib.debugPrint
local strsplit = strsplit
local tonumber = tonumber
local UnitName = UnitName
local unpack = unpack

--Return the duration of a spell.
function lib:GetDuration(spellID, srcGUID)
	if self.auraInfo[spellID] then
		--Check if we've seen that GUID has a duration that doesn't match our base.
		if srcGUID then
			if self.GUIDDurations[srcGUID.."-"..spellID] then
--~ 				debugPrint("GetDuration", srcGUID, spellID, lib.GUIDDurations[srcGUID.."-"..spellID])
				return self.GUIDDurations[srcGUID.."-"..spellID]
			end
		end
	
		local duration, debuffType = strsplit(";", self.auraInfo[spellID])
		return tonumber(duration or 0)
	end
	return nil
end

--Return the debuff type of a spell.
function lib:GetDebuffType(spellID)
	if self.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", self.auraInfo[spellID])
		if debuffType then
			if debuffType == 0 then
				return "None"
			else
				return self.debuffTypes[debuffType] or "unknown"
			end
		end
		return "None"
	end
	return nil
end

function lib:GetNumGUIDAuras(GUID)
	if self.GUIDAuras[GUID] then
		return #self.GUIDAuras[GUID]
	end
	return 0
end

function lib:GetGUIDAuras(GUID)
	if self.GUIDAuras[GUID] then
		return unpack(self.GUIDAuras[GUID])
	end
	return nil
end
--~ name, rank, icon, count,  dispelType, duration, expires, caster,  isStealable, shouldConsolidate, spellID  = UnitAura("unit", index [,  "filter"]) or UnitAura("unit", i)
function lib:GUIDAura(dstGUID, spellID)
	local key = dstGUID.."-"..spellID
	local count = self.GUIDAuraData_stackCount[key] or 1
	local debuffType = self.GUIDAuraData_debuffType[key] or "none"
	local duration = self.GUIDAuraData_duration[key] or 0
	local expires = self.GUIDAuraData_expirationTime[key] or 0
	local isDebuff = lib.GUIDAuraData_isDebuff[key]
	
	return count, debuffType, duration, expires, isDebuff
end